//
//  MFUtil.h
//  MFAuthLib
//
//  Created by MicroFocus on 5/22/17.
//  Copyright © 2017 Microfocus. All rights reserved.
//

#import <Foundation/Foundation.h>

@interface MFUtil : NSObject

+ (NSData *) httpBodyForParams:(NSDictionary *) params;

/*! @brief Get the URL Encoded or Percent Escaped String.
 The allowed characters are "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-._~".
 @param str input string to be converted.
 @return URL Encoded String.
  */
+ (NSString *) percentEscapeString:(NSString *) str;

/*! @brief Get the Base64 Encoded String.
 @param str input string to be converted.
 @return Base64 Encoded String.
 */
+ (NSString *) base64Encode:(NSString *) str;

/*! @brief Get the Base64 Decoded String.
 @param base64EncodedStr input string to be converted.
 @return Base64 Decoded String.
 */
+ (NSString *) base64Decode:(NSString *) base64EncodedStr;

/*! @brief Get the unique device Id.
 It uses the iOS identifierForVendor @link https://developer.apple.com/reference/uikit/uidevice/1620059-identifierforvendor
 This is the same UDID for all apps from same vendor running on same device.
 A different value is returned for apps on the same device that come from different vendors and for apps even with
 same vendor but on different devices.
 The v
 */
+ (NSString *) vendorDeviceID;

/*! @brief Get the SDK Version String
 */
+ (NSString *) sdkVersion;

@end
