//
//  ViewController.m
//  TApp2
//
//  Created by Microfocus on 2/16/17.
//  Copyright © 2017 Microfocus. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "NSDictionary+CaseInsensitiveLookup.h"

#import "ViewController.h"
#import "AppDelegate.h"
#import "AccountConfig.h"

#import <MFAuthLib/MFAuthLib.h>
#import <MFAuthLib/MFConstants.h>

@class MFAuthService;


static NSString *const kIssuerUrlKey       = @"IssuerUrl";

/*! @brief The OIDC issuer from which the configuration will be discovered.
 */
static NSString *const kIssuer = @"https://prvdvnam625.namdom055.lab/nidp/oauth/nam";

/*! @brief The OAuth client ID.
 @discussion The client should be registered with the "native" type.
 */

static NSString *const kClientID = @"1a6b54a1-c205-475a-9346-435da22ae70d";
static NSString *const kClientSecret = @"1snB98w_4t0EUOEzURov104j6etni1E2rZ2Vyd4XTvNeuJUX5kQEpYxMh_9sKaC-5zE84MU-OFdDJ0oSErN_kA";

/*! @brief The OAuth redirect URI for the client @c kClientID.
 @discussion This scheme must be registered as a scheme in the project's Info
 property list ("CFBundleURLTypes" plist key). Any path component will work, we use
 'oauthredirect' here to help disambiguate from any other use of this scheme.
 */
static NSString *const kRedirectURI =
@"myapp2://oauthredirect/";


static NSString *const kScopeOpenID = @"openid";
static NSString *const kScopeProfile = @"profile";
static NSString *const kScopeEmail = @"email";
static NSString *const kScopeAddress = @"address";
static NSString *const kScopePhone = @"phone";

UIAlertController *alertController;
AccountConfig *accountConfig;
NSString *sProvider;

BOOL inflight = NO;

@interface ViewController ()

@end


@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    //Clear keychain on first run in case of reinstallation
    if (![[NSUserDefaults standardUserDefaults] objectForKey:@"FirstRun"]) {
        // Delete values from keychain
        [MFAuthService removeKeychainEntry];
        [[NSUserDefaults standardUserDefaults] setValue:@"1strun" forKey:@"FirstRun"];
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    
    
#if !defined(NS_BLOCK_ASSERTIONS)
    
    // NOTE:
    //
    // To run this sample, you need to register your own iOS client on NAM admin portal
    // and update three configuration points in the sample: kClientID and kRedirectURI constants in Example ViewController.m and the URI scheme in Info.plist (URL Types -> Item 0 -> URL Schemes -> Item 0).
    
    NSAssert(![kClientID isEqualToString:@"YOUR_CLIENT_ID"],
             @"Update kClientID with your own client ID. ");
    
    NSAssert(![kRedirectURI isEqualToString:@"YOUR_APP_SCHEME://oauthredirect"],
             @"Update kRedirectURI with your own redirect URI. ");
    
    // verifies that the custom URI scheme has been updated in the Info.plist
    NSArray __unused* urlTypes =
    [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleURLTypes"];
    NSAssert([urlTypes count] > 0, @"No custom URI scheme has been configured for the project.");
    NSArray *urlSchemes =
    [(NSDictionary *)[urlTypes objectAtIndex:0] objectForKey:@"CFBundleURLSchemes"];
    NSAssert([urlSchemes count] > 0, @"No custom URI scheme has been configured for the project.");
    NSString *urlScheme = [urlSchemes objectAtIndex:0];
    
    NSAssert(![urlScheme isEqualToString:@"YOUR_APP_SCHEME"],
             @"Configure the URI scheme in Info.plist (URL Types -> Item 0 -> URL Schemes -> Item 0) "
             "with the scheme of your redirect URI.");
    
#endif // !defined(NS_BLOCK_ASSERTIONS)
    
    _logTextView.layer.borderColor = [UIColor colorWithWhite:0.8 alpha:1.0].CGColor;
    _logTextView.layer.borderWidth = 1.0f;
    _logTextView.alwaysBounceVertical = true;
    _logTextView.textContainer.lineBreakMode = NSLineBreakByCharWrapping;
    _logTextView.text = @"";
    
    sProvider = [self loadConfig];
    if (sProvider)
        [self initAuthServiceForAccount:accountConfig];
    
    [self updateUI];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    if ( (!sProvider) && (!inflight) ) {
        [self askProvider];
    }
    [self logMessage:@"vid=%@", [[[UIDevice currentDevice] identifierForVendor] UUIDString]];
}

/*! @brief Refreshes UI, typically called after the auth state changed.
 */
- (void)updateUI {
    _userinfoButton.enabled = [_authService isAuthorized];
    _clearAuthStateButton.enabled = [_authService isAuthorized];
    //_codeExchangeButton.enabled = _authState.lastAuthorizationResponse.authorizationCode && !_authState.lastTokenResponse;
    // dynamically changes authorize button text depending on authorized state
    if (![_authService isAuthorized]) {
        [_authAutoButton setTitle:@"Authorize" forState:UIControlStateNormal];
        [_authAutoButton setTitle:@"Authorize" forState:UIControlStateHighlighted];
        [_authManual setTitle:@"Authorize (Manual)" forState:UIControlStateNormal];
        [_authManual setTitle:@"Authorize (Manual)" forState:UIControlStateHighlighted];
    } else {
        [_authAutoButton setTitle:@"Re-authorize" forState:UIControlStateNormal];
        [_authAutoButton setTitle:@"Re-authorize" forState:UIControlStateHighlighted];
        [_authManual setTitle:@"Re-authorize (Manual)" forState:UIControlStateNormal];
        [_authManual setTitle:@"Re-authorize (Manual)" forState:UIControlStateHighlighted];
    }
}

- (void)authState:(OIDAuthState *)state didEncounterAuthorizationError:(nonnull NSError *)error {
    [self logMessage:@"Received authorization error: %@", error];
}

- (BOOL) isRootUrl:(NSURL *) providerUrl {
    NSArray *arPaths = [providerUrl pathComponents];
    if ([arPaths count] < 2) {
        return YES;
    }
    return NO;
}

- (void) setProvider:(NSString *) sIssuer {
    sProvider = sIssuer;
}

- (void) setAuthService:(MFAuthService * _Nullable)authService {
    _authService = authService;
}

- (void) getOAuthDataFromServer:(NSString *) svrUrl {
    NSError *error;
    NSMutableDictionary* jsonDict;
    
    [self logMessage:@"Getting OAuth config data from: %@", svrUrl];
    
    NSData* data = [NSData dataWithContentsOfURL:[NSURL URLWithString:svrUrl]];
    if (data) {
        jsonDict = [NSJSONSerialization JSONObjectWithData:data
                                                   options:kNilOptions
                                                     error:&error];
    }
    if (jsonDict) {
        if ([jsonDict objectForCaseInsensitiveKey:@"ProviderUrl"]) {
            accountConfig = [[AccountConfig alloc]
                          initWithProviderUrl:[jsonDict objectForCaseInsensitiveKey:@"ProviderUrl"]
                          clientId:[jsonDict objectForCaseInsensitiveKey:@"ClientID"]
                          clientSecret:[jsonDict objectForCaseInsensitiveKey:@"ClientSecret"]
                          redirectURI:[jsonDict objectForCaseInsensitiveKey:@"RedirectURI"]
                          scope:[jsonDict objectForCaseInsensitiveKey:@"Scope"]
                          accountName:[jsonDict objectForCaseInsensitiveKey:@"AccountName"]
                          ];
        } else if ([jsonDict objectForCaseInsensitiveKey:@"DiscoveryUrl"]) {
            accountConfig = [[AccountConfig alloc]
                          initWithDiscoveryUrl:[jsonDict objectForCaseInsensitiveKey:@"DiscoveryUrl"]
                          revocationEndpoint:[jsonDict objectForCaseInsensitiveKey:@"RevocationUrl"]
                          browserLogoutUrl:[jsonDict objectForCaseInsensitiveKey:@"LogoutUrl"]
                          clientId:[jsonDict objectForCaseInsensitiveKey:@"ClientID"]
                          clientSecret:[jsonDict objectForCaseInsensitiveKey:@"ClientSecret"]
                          redirectURI:[jsonDict objectForCaseInsensitiveKey:@"RedirectURI"]
                          accountName:[jsonDict objectForCaseInsensitiveKey:@"AccountName"]
                          ];
        } else {
            accountConfig = [[AccountConfig alloc]
                          initWithAuthzEndpoint:[jsonDict objectForCaseInsensitiveKey:@"AuthzUrl"]
                          tokenEndpoint:[jsonDict objectForCaseInsensitiveKey:@"TokenUrl"]
                          revocationEndpoint:[jsonDict objectForCaseInsensitiveKey:@"RevocationUrl"]
                          userInfoEndpoint:[jsonDict objectForCaseInsensitiveKey:@"UserinfoUrl"]
                          browserLogoutUrl:[jsonDict objectForCaseInsensitiveKey:@"LogoutUrl"]
                          clientId:[jsonDict objectForCaseInsensitiveKey:@"ClientID"]
                          clientSecret:[jsonDict objectForCaseInsensitiveKey:@"ClientSecret"]
                          redirectURI:[jsonDict objectForCaseInsensitiveKey:@"RedirectURI"]
                          scope:[jsonDict objectForCaseInsensitiveKey:@"Scope"]
                          accountName:[jsonDict objectForCaseInsensitiveKey:@"AccountName"]
                          ];
        }
        [accountConfig setTestPR1:[jsonDict objectForCaseInsensitiveKey:@"TestPR"] ];
    }
    [self logMessage:@"Got Configuration: %@", jsonDict];
}

- (void) askProvider
{
    alertController = [UIAlertController alertControllerWithTitle:@"Provider" message:@"Enter location of Json config file  with Provider Details" preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction * action) {
        [self logMessage:@"User Cancelled Login."];
        [alertController dismissViewControllerAnimated:YES completion:nil];
    }];
    [alertController addAction:cancelAction];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * action) {
        NSString *sUrl = alertController.textFields.firstObject.text;
        if (![sUrl hasPrefix:@"http"]){
            sUrl = [@"http://" stringByAppendingString:sUrl];
        }
        [self getOAuthDataFromServer:sUrl];
        if (accountConfig) {
            [self initAuthServiceForAccount:accountConfig];
        } else {
            [self logMessage:@"Error initializing Config from %@.", sUrl];
        }
    }];
    [alertController addAction:okAction];
    
    [alertController addTextFieldWithConfigurationHandler:^(UITextField *textField) {
        textField.keyboardType = UIKeyboardTypeURL;
        textField.placeholder = @"http://www.example.com/provider.json";
        //textField.text = @"http://";
    }];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void) initAuthServiceForAccount:(AccountConfig *) acctConfig {
    if (!acctConfig) {
        return;
    }
    
    __block MFAuthService *authSvc = nil;
    NSString *sProviderUrl = [accountConfig getProviderUrl];
    if (!sProviderUrl) {
        NSString *sDiscoveryUrl = [accountConfig getDiscoveryUrl];
        if (sDiscoveryUrl) {
            [MFAuthService discoverServiceConfiguration:[NSURL URLWithString:sDiscoveryUrl]
                                             completion:^(MFServiceConfiguration *serviceConfiguration, NSError *error) {
                                                 if (serviceConfiguration) {
                                                     NSURL *issuerUrl = serviceConfiguration.discoveryDocument.issuer;
                                                     [self setProvider:[issuerUrl absoluteString]];
                                                     serviceConfiguration.userinfoEndpoint = serviceConfiguration.discoveryDocument.userinfoEndpoint;
                                                     
                                                     serviceConfiguration.revocationEndpoint = [NSURL URLWithString:[accountConfig getRevocationUrl] ];
                                                     
                                                     serviceConfiguration.browserLogoutEndpoint = [NSURL URLWithString:[accountConfig getBrowserLogoutUrl] ];
                                                     
                                                     authSvc = [[MFAuthService alloc]
                                                                     initWithServiceConfig:serviceConfiguration];
                                                     [self setAuthService:authSvc];
                                                 } else {
                                                     [self logMessage:@"Error fetching discovery document %@", error];
                                                 }
                                             }];
        } else {
            if (!([accountConfig getAuthzUrl]) || !([accountConfig getAuthzUrl]) ) {
                [self logMessage:@"No OIDC Discovery Or OAuth Endpoints found in the config."];
            } else {
                authSvc = [[MFAuthService alloc]
                                initWithAuthzEndpoint:[NSURL URLWithString:[accountConfig getAuthzUrl] ]
                                tokenEndpoint:[NSURL URLWithString:[accountConfig getTokenUrl] ]
                                revocationEndpoint:[NSURL URLWithString:[accountConfig getRevocationUrl] ]
                                userinfoEndpoint:[NSURL URLWithString:[accountConfig getUserinfoUrl] ]
                                browserLogoutEndpoint:[NSURL URLWithString:[accountConfig getBrowserLogoutUrl] ]
                                ];
                [self setAuthService:authSvc];
                [self setProvider:[accountConfig getTokenUrl] ];
            }
        }
    } else {
        if ([self isRootUrl:[NSURL URLWithString:sProviderUrl] ] ) {
            // Provider Base (Root) Url was provided
            authSvc = [[MFAuthService alloc] initWithProvider:[NSURL URLWithString:sProviderUrl]];
        } else {
            // IssuerUrl was provided
            NSURL *logoutUrl = nil;
            NSString *sLogoutUrl = [accountConfig getBrowserLogoutUrl];
            if (sLogoutUrl) {
                logoutUrl = [NSURL URLWithString:sLogoutUrl];
            }
            authSvc = [[MFAuthService alloc] initWithIssuerUrl:[NSURL URLWithString:sProviderUrl] logoutUrl:logoutUrl ];
        }
        [self setAuthService:authSvc];
        [self setProvider:[accountConfig getProviderUrl] ];
    }
}

- (IBAction)authWithAutoCodeExchange:(nullable id)sender {
    
    if (!_authService) {
        [self logMessage:@"*** AuthService is not properly initilized. try restarting the app."];
        return;
    }
    
    if ([_authService isAuthorized]) {
        [self logMessage:@"Already Authorized"];
        return;
    }
    
    if ( !accountConfig ) {
        [self askProvider];
        return;
    }
    
    AppDelegate *appDelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;
    NSArray *arScopes = [[accountConfig getScope] componentsSeparatedByString:@" "];
    
    inflight = YES;
    /*
    NSMutableDictionary <NSString *, NSString *> *additionalParams = [ @{
                             @"prompt" : @"login"
                             } mutableCopy];
    */
    appDelegate.currentAuthorizationFlow = [_authService signInWithClientId:[accountConfig getClientId]
                        clientSecret:[accountConfig getClientSecret]
                              scopes:arScopes
                         redirectURL:[ NSURL URLWithString:[accountConfig getRedirectUri] ]
            presentingViewController:self
                           withBlock:^(BOOL success, NSError * _Nullable error) {
                               //appDelegate.currentAuthorizationFlow = currentAuthFlowSession;
                               if (success) {
                                   [self logMessage:@"New Access Token Expiration: %@", [_authService getAccessTokenExpiryTimeString] ];
                                   [self saveConfig:sProvider];
                               } else {
                                   [self logMessage:@"Error during OAuth flow %@", [error localizedDescription]];
                               }
                               inflight = NO;
                               [self updateUI];
                           }];
    if (!appDelegate.currentAuthorizationFlow) {
        inflight = NO;
    }
}


- (IBAction)authNoCodeExchange:(id)sender {
    //not implemented
    [self logMessage:@"Not Implemented"];
}

- (IBAction)codeExchange:(id)sender {
    //not implemented
    [self logMessage:@"Not Implemented"];
}


- (IBAction)userinfo:(nullable id)sender {
    [self logMessage:@"Access Token Expiration: %@", [_authService getAccessTokenExpiryTimeString] ];
    [_authService getUserInfo:^(NSDictionary *_Nullable dictUserdata, NSError *_Nullable error) {
        dispatch_async(dispatch_get_main_queue(), ^() {
            if (error) {
                [self logMessage:@"HTTP request failed %@", error];
                return;
            } else {
                [self logMessage:@"New Access Token Expiration: %@ \n\t", [_authService getAccessTokenExpiryTimeString] ];
                [self logMessage:@"Successs: %@", dictUserdata];
            }
        });
    }];
    
}

- (IBAction)clearAuthState:(nullable id)sender {
    [self removeConfig];
    [_authService logout:self];
    [self clearLog:nil];
    [self updateUI];
}

- (IBAction)deleteConfigButton:(nullable id)sender {
    [self removeConfig];
    [self logMessage:@"Config Deleted. Config: %@", [accountConfig getAccountName]];
    [self askProvider];
}

- (IBAction)clearLog:(nullable id)sender {
    _logTextView.text = @"";
}

- (void) saveConfig:(NSString *) providerUrl
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSData *data  = nil;
    if (accountConfig) {
        /*
        if (![accountConfig saveAccountConfig]) {
            [self logMessage:@"** Error saving account config"];
        }
         */
      data = [NSKeyedArchiver archivedDataWithRootObject:accountConfig];
    }
    // Save the Provider Url from the text field into the prefs
    [defaults setObject:data forKey:@"accountConfig"];
    [defaults setObject:providerUrl forKey:kIssuerUrlKey];
    [defaults synchronize];
}

- (NSString *) loadConfig
{
    /*
    accountConfig = [AccountConfig loadAccountConfig];
    if (!accountConfig) {
        [self logMessage:@"No account config loaded"];
    }
     */
    // Load the provider Url from the prefs into the text fields
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSData *data = [defaults objectForKey:@"accountConfig"];
    accountConfig = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    return [defaults stringForKey:kIssuerUrlKey];
}

- (void) removeConfig
{
    /*
    if (![accountConfig removeAccountConfig]) {
        [self logMessage:@"** Error removing account config"];
    }
     */
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults removeObjectForKey:@"accountConfig"];
    [defaults removeObjectForKey:kIssuerUrlKey];
    [defaults synchronize ];
    accountConfig = nil;
    sProvider = nil;
}


/*! @brief Logs a message to stdout and the textfield.
 @param format The format string and arguments.
 */
- (void)logMessage:(NSString *)format, ... NS_FORMAT_FUNCTION(1,2) {
    // gets message as string
    va_list argp;
    va_start(argp, format);
    NSString *log = [[NSString alloc] initWithFormat:format arguments:argp];
    va_end(argp);
    
    // outputs to stdout
    NSLog(@"%@", log);
    
    // appends to output log
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    dateFormatter.dateFormat = @"hh:mm:ss";
    NSString *dateString = [dateFormatter stringFromDate:[NSDate date]];
    _logTextView.text = [NSString stringWithFormat:@"%@%@%@: %@",
                         _logTextView.text,
                         ([_logTextView.text length] > 0) ? @"\n" : @"",
                         dateString,
                         log];
}

@end

